package com.qmth.teachcloud.report.api;

import com.alibaba.fastjson.JSON;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.report.business.bean.dto.printOpen.*;
import com.qmth.teachcloud.report.business.service.CallPrintOpenApiService;
import io.swagger.annotations.*;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.List;

/**
 * @Description:
 * @Author: CaoZixuan
 * @Date:
 */
@Api(tags = "测试请求分布式印刷开放接口Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/test/call")
@Validated
public class TestCallPrintOpenApiController {
    @Resource
    CallPrintOpenApiService callPrintOpenApiService;

    @ApiOperation(value = "试卷配置")
    @ApiResponses({@ApiResponse(code = 200, message = "试卷配置", response = Object.class)})
    @RequestMapping(value = "/paper_config", method = RequestMethod.POST)
    @Aac(auth = BOOL.FALSE)
    public Result paperConfig(@ApiParam(value = "考试id", required = true) @RequestParam Long thirdExamId,
                              @ApiParam(value = "科目编号", required = true) @RequestParam String courseCode) throws IOException {
        PaperConfig paperConfig = callPrintOpenApiService.callPaperConfig(thirdExamId, courseCode);
        System.out.println(JSON.toJSONString(paperConfig));
        return ResultUtil.ok();
    }

    @ApiOperation(value = "维度配置")
    @ApiResponses({@ApiResponse(code = 200, message = "试卷配置", response = Object.class)})
    @RequestMapping(value = "/paper_dimension", method = RequestMethod.POST)
    @Aac(auth = BOOL.FALSE)
    public Result paperDimension(@ApiParam(value = "考试id", required = true) @RequestParam Long thirdExamId,
                                 @ApiParam(value = "科目编号", required = true) @RequestParam String courseCode) throws IOException {
        List<PaperDimension> paperDimensionList = callPrintOpenApiService.callPaperDimension(thirdExamId, courseCode);
        System.out.println(JSON.toJSONString(paperDimensionList));
        return ResultUtil.ok();
    }

    @ApiOperation(value = "蓝图配置")
    @ApiResponses({@ApiResponse(code = 200, message = "试卷配置", response = Object.class)})
    @RequestMapping(value = "/paper_struct", method = RequestMethod.POST)
    @Aac(auth = BOOL.FALSE)
    public Result paperStruct(@ApiParam(value = "考试id", required = true) @RequestParam Long thirdExamId,
                              @ApiParam(value = "科目编号", required = true) @RequestParam String courseCode) throws IOException {
        List<PaperStructure> paperStructureList = callPrintOpenApiService.callPaperStruct(thirdExamId, courseCode);
        System.out.println(JSON.toJSONString(paperStructureList));
        return ResultUtil.ok();
    }

    @ApiOperation(value = "模块配置")
    @ApiResponses({@ApiResponse(code = 200, message = "试卷配置", response = Object.class)})
    @RequestMapping(value = "/paper_module", method = RequestMethod.POST)
    @Aac(auth = BOOL.FALSE)
    public Result paperModule(@ApiParam(value = "考试id", required = true) @RequestParam Long thirdExamId,
                              @ApiParam(value = "科目编号", required = true) @RequestParam String courseCode) throws IOException {
        List<PaperEvaluation> paperEvaluationList = callPrintOpenApiService.callPaperModule(thirdExamId, courseCode);
        System.out.println(JSON.toJSONString(paperEvaluationList));
        return ResultUtil.ok();
    }

    @ApiOperation(value = "考生成绩")
    @ApiResponses({@ApiResponse(code = 200, message = "考生成绩", response = Object.class)})
    @RequestMapping(value = "/exam_student_score", method = RequestMethod.POST)
    @Aac(auth = BOOL.FALSE)
    public Result examStudentScore(@ApiParam(value = "考试id", required = true) @RequestParam Long thirdExamId,
                                   @ApiParam(value = "科目编号", required = true) @RequestParam String courseCode) throws IOException {
        List<ExamStudentScore> examStudentScoreList = callPrintOpenApiService.callExamStudentScore(thirdExamId, courseCode);
        System.out.println(JSON.toJSONString(examStudentScoreList));
        return ResultUtil.ok();
    }
}
